#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar  4 18:33:32 2021

@author: Mathilde
"""

import numpy as np

import matplotlib.pyplot as plt

### Q2

def euler(n):
    """
    La fonction prend en entrée un nombre n de points à construire et 
    résout l'équation y'=y**2 sur l'intervalle [0,3/2] avec y0=1/2
    """
    pas = (1.5-0)/(n-1)
    #pas est le pas de la méthode d'Euler
    Liste_y=[0.5]
    #On va stocker les ordonnées dans Liste_y
    for i in range(n-1):
        y=Liste_y[i]+(Liste_y[i])**2*pas
        # On applique la méthode d'Euler pour construire le point 
        #suivant et on le rajoute dans la liste des ordonnées
        Liste_y=Liste_y+[y]
    return Liste_y

### Ci-dessous, voici une solution plus générale qui peut s'adapter à 
### n'importe quelle équation différentielle de la forme y'=fonction(y)
### sur un intervalle [x0,xmax] quelconque et avec y0 quelconque


def carre(x):
    #la fonction carre renvoie le carré d'un nombre (entier ou flottant)
    return(x**2)

def euler_general(x0,xmax,y0,fonction,n):
    """
    La fonction prend en entrée un intervalle [x0;xmax] sur lequel
    on veut résoudre l'équation différentielle. 
    Elle prend aussi n, le nombre de points qu'on souhaite obtenir
    La fonction appelée "fonction" définit l'équation différentielle
    à résoudre avec f'=fonction(f)
    """
    pas = (xmax-x0)/(n-1)
    #pas est le pas de la méthode d'Euler
    Liste_y=[y0]
    #On va stocker les ordonnées dans Liste_y
    for i in range(n-1):
        y=Liste_y[i]+fonction(Liste_y[i])*pas
        Liste_y=Liste_y+[y]
    return Liste_y

### Q3

def affiche_Euler(n):
    """
    Cette fonction affiche la courbe obtenue avec la 
    méthode d'Euler pour l'équation différentielle y'=y
    avec la condition initiale y(0)=1
    """
    abscisse=np.linspace(0,2,n)
    ordonnee_approx=euler_general(0,3/2,1/2,carre,n)
    plt.plot(abscisse,ordonnee_approx,"-b")
  
### Q5
    
def affiche_exacte(n):
    #Cette fonction affiche la courbe de la fonction
    #obtenue en résolvant l'équation différentielle y'=y**2
    #avec la condition initiale y(0)=1/2 sur l'intervalle [0,3/2]
    # càd la fonction f(x) = 1/(2-x)
    abscisse=np.linspace(0,3/2,n)
    ordonnee_exacte=1/(2-abscisse)
    plt.plot(abscisse,ordonnee_exacte,"-g")


### Q6
def affiche_compare(n):
    #Cette fonction affiche la courbe obtenue avec la 
    #méthode d'Euler ainsi que la courbe de la fonction
    #obtenue en résolvant l'équation différentielle y'=y**2
    #avec la condition initiale y(0)=1/2
    abscisse=np.linspace(0,3/2,n)
    ordonnee_approx=euler_general(0,3/2,1/2,carre,n)
    ordonnee_exacte=1/(2-abscisse)
    plt.plot(abscisse,ordonnee_approx,"-b",abscisse,ordonnee_exacte,"-g")
    
### Q7
def erreur(n):
    #renvoie l'erreur maximale pour n points
    ecart=0
    x=np.linspace(0,1.5,n)
    y_approx=euler(0,1.5,0.5,carre,n)
    y_exact=1/(2-x)
    for i in range(n):
        diff=abs(y_approx[i]-y_exact[i])
        if diff>ecart:
            ecart=diff
    return ecart    

### Q8

def solution_exacte(x):
    # x est un nombre
    # renvoie l'image de x par x -> 1/(2-x)
    return(1/(2-x))    

def affiche_erreur():
    # procédure qui calcule l'erreur max commise par la méthode d'Euler
    # pour n points avec n qui varie de 10 à 1000 par pas de 10
    # provoque l'affichage de l'erreur en fonction de n    
    N=range(10,1000,10)
    erreur=[]
    for n in N:
        temps=np.linspace(0,1.5,n)
        exact=[solution_exacte(i) for i in temps]
        approx=euler_general(0,.5,1.5,carre,n)
        erreur_pour_n=[abs(e-a) for e,a in zip(exact,approx)]
        erreur.append(max(erreur_pour_n))
        
    plt.plot(N,erreur,'-y')